'' Lander02 - Simple Moon Lander Simulation
''
'' Author: MSrobots
'' Version: 2
'' Date: 2018-09-16
''
'' Copyright (c) 2018 Michael Sommer
'' See end of file for terms of use.
''
CON                                 
                                                                              
  _clkmode = XTAL1|PLL16X                                                                    
  _xinfreq = 5_000_000

  gravity = 100 'The rate in which the spaceship descents in free fall (in ten seconds) 
  maxstep = 9 ' numbers of gridlines
  
OBJ
  wfs    : "WFsystem"           ' Basic WF support

VAR

  long state[4]                 'State of each Game
  long oldstate[4]              'old State for HELP
  long curMapF[4]               'address of filename of map
  long curMapW[4]               'width  in map for HELP and SCORE
  long curMapH[4]               'height in map for HELP and SCORE
  long curMapX[4]               'xpos   in map for HELP and SCORE
  long curMapY[4]               'ypos   in map for HELP and SCORE
  long curMapC[4]               'Color  in map for HELP and SCORE
  
  long height[4]                'The height of the spaceship.
  long speed[4]                 'The speed of the spaceship.
  long burn[4]                  'The fuel which gets burned this step 
  long tensec[4]                'The time the flight is running for. (in ten second steps) 
  long fuel[4]                  'The fuel you have left. (kilogram) 
  long prevheight[4]            'The previous height to compare with actual. (for the colored digits) 
  long stepcount[4]             'Counts the steps passed since last output of the column names
  long scoreDead[4]             'Counts Dead results since loading of the game
  long scoreCrash[4]            'Counts Crash results since loading of the game
  long scoreLand[4]             'Counts Land results since loading of the game

DAT '                                ###   THE MAIN LOOP   ###
PUB main | s, ch, sw, sh, sx, sy 
's=screen, i/j=gp, s0/s1=secs time 0/1, ch=char, rot=rotate, 'ch=kbd character
  wfs.initSystem     'start up system: video, keyboards, sd card, etc.
  splashScreens
  repeat s from 0 to 3
    scoreDead[s] := 0
    scoreCrash[s] := 0
    scoreLand[s] := 0
    writeAllscores(s)
    curMapY[s] := 0
    startGame(s)
      
  repeat 'main loop, infinite
    s := (s + 1) // 4       'cycle among the keyboards (0..3, wrapping around)
    ch := wfs.getKey(s)            'get key in keyboard buffer, if present; 0 = no key
    if ch <> 0                       'check if there's a buffered keystroke for curr. kbd.        
      if state[s] < 1
        if state[s]<0
          startGame(s)
        else
          state[s] := 1           ' Start Game 
          drawScreen(s)           ' output new screen
          drawState(s)            ' output current values
      elseif state[s]==2          ' showing HELP
        if curMapF[s] == @ScoreMap 
          sw := 36
          sh := 11
          sx := 2
          sy := 2
        else
          sw := 34
          sh := 11
          sx := 3
          sy := 2         
        case ch
          232:  'up
            if curMapY[s] > -(sh-1)
              curMapY[s]--
              wfs.blitMap(s, curMapF[s], curMapW[s], curMapH[s], curMapX[s], curMapY[s], sw, sh, sx, sy, curMapC[s], ".", wfs#GR)
          228: 'left
            if curMapX[s] > -(sw-1)
              curMapX[s]--
              wfs.blitMap(s, curMapF[s], curMapW[s], curMapH[s], curMapX[s], curMapY[s], sw, sh, sx, sy, curMapC[s], ".", wfs#GR)
          230:  'right
            if curMapX[s] < (curMapW[s]-1)
              curMapX[s]++
              wfs.blitMap(s, curMapF[s], curMapW[s], curMapH[s], curMapX[s], curMapY[s], sw, sh, sx, sy, curMapC[s], ".", wfs#GR)
          226:  'down          
            if curMapY[s] < (curMapH[s]-1)
              curMapY[s]++ 
              wfs.blitMap(s, curMapF[s], curMapW[s], curMapH[s], curMapX[s], curMapY[s], sw, sh, sx, sy, curMapC[s], ".", wfs#GR)
          other:
            if curMapF[s] <> @ScoreMap 
               curMapY[s] := 0   
            state[s] := oldstate[s]
            StoreTempS := s+48
            wfs.restoreScreen(s, @StoreTemp,-2)
      else      
        case ch                   ' GAME
          8:
            truncateNumber(s)
          13:
            nextStep(s)
          48..58:
            addNumber(s,ch)
          200:
            truncateNumber(s)
          203:
            wfs.bootPartition(string("/GAMEMENU/GAMEMENU.BIN")) 'launch GameMenu
          208:
            'F1 - Help
            showHelp(s)
          209:
            'F2 - Score
            curMapY[s] := 0
            showScore(s)
      
PUB version
RETURN 2
DAT                           'Support Methods for MAIN LOOP Method
PRI showHelp(s)
  oldstate[s] :=  state[s]
  state[s] := 2
  StoreTempS := s+48
  wfs.storeScreen(s, @StoreTemp,-2)
  curMapF[s] := @HelpFile
  curMapC[s] := wfs#RG' file contains just Text, so Colors are fixed
  curMapW[s] := 36
  curMapH[s] := wfs.fileSize(@HelpFile) / curMapW[s]
  curMapX[s] := 0
  curMapY[s] := 0
  wfs.blitMap(s, curMapF[s], curMapW[s], curMapH[s], curMapX[s], curMapY[s], 34, 11, 3, 2, curMapC[s], ".", wfs#GR)
   
PRI showScore(s)
  oldstate[s] :=  state[s]
  state[s] := 2
  StoreTempS := s+48
  wfs.storeScreen(s, @StoreTemp,-2)
  curMapF[s] := @ScoreMap
  curMapC[s] := -2 ' file contains Colors
  curMapW[s] := 36
  curMapH[s] := (wfs.fileSize(@ScoreMap) / curMapW[s]) / 2
  curMapX[s] := 0
  'curMapY[s] := 0
  wfs.blitMap(s, curMapF[s], curMapW[s], curMapH[s], curMapX[s], curMapY[s], 36, 11, 2, 2, curMapC[s], ".", wfs#GR)
   
PRI splashScreens | s, i 
  'show splash momentarily
  wfs.beep
  StoreScreensN := "0"
  repeat s from 0 to 3
    wfs.restoreScreen(s, @StoreScreens,-2)
  waitcnt(clkfreq*5+cnt)
  wfs.beep

PRI startGame(s) | row
  state[s] := 0
  speed[s] := 1000
  height[s] := cnt & $FFFF
  ?height[s]
  height[s] := ((height[s] & $FFFF) // 15000) + 4000
  fuel[s] := 12000
  tensec[s] := 0
  burn[s] := 0
  prevheight[s] := height[s]
  stepcount[s] := 1
    
  if (scoreDead[s]==0) and (scoreCrash[s]==0) and (scoreLand[s]==0)
    StoreScreensN := "1"
    wfs.restoreScreen(s, @StoreScreens,-2)
  else
    state[s] := 1          ' output new screen
    drawScreen(s)           ' output new screen
    drawState(s)            ' output current values
    showScore(s) 

PRI drawScreen(s)
  StoreScreensN := "2"
  wfs.restoreScreen(s, @StoreScreens,-2)
  
PRI drawState(s) | row
  row := stepcount[s]+3
  wfs.write(s, 0, row, wfs#GK, string("       |       |       |       |       "))
  drawNumber6(s, 0, row, wfs#GK, tensec[s])
  if speed[s] <0
    drawNumber6(s, 8, row, wfs#RK, speed[s])
  else  
    drawNumber6(s, 8, row, wfs#GK, speed[s])
  drawNumber6(s, 16, row, wfs#GK, fuel[s])
  if height[s] > prevheight[s]
    drawNumber6(s, 24, row, wfs#RK, height[s])
  else
    drawNumber6(s, 24, row, wfs#GK, height[s])
  drawNumber6(s, 32, row, wfs#GK, burn[s])

PRI drawNumber6(s, c, r, k, n) | col, val, ch, sign
   val := n
   if val<0
     sign := "-"
     val := -val
   else
     sign := " "
   col := c + 5
   repeat 5
     ch := (val // 10) + 48
     val := val / 10
     wfs.putChar(s, col, r, k, ch)
     col--
   wfs.putChar(s, col, r, k, sign)

PRI addNumber(s, ch)
  burn[s] := (burn[s]*10)+ch-48
  if burn[s] > 250
    burn[s] := 250
  if burn[s]>fuel[s]
     burn[s] := fuel[s]
  drawNumber6(s, 32, 14, wfs#RG, burn[s])

PRI truncateNumber(s)
  burn[s] := burn[s]/10
  drawNumber6(s, 32, 14, wfs#RG, burn[s])

PRI nextStep(s)
  prevheight[s] := height[s]
  speed[s] += gravity - burn[s]                       ' calculate(height, speed, burn, gravity);
  height[s] -= speed[s]
  fuel[s] -= burn[s]
  tensec[s] += 10
  stepcount[s]++
  if stepcount[s] > maxstep
    stepcount[s] := 1
    drawScreen(s)
  drawState(s)
  burn[s] := 0
  drawNumber6(s, 32, 14, wfs#RG, burn[s])
  if height[s]<1
    if speed[s]>10
      wfs.repChar(s, 0, 13, wfs#KR, 32, 40)
      wfs.write(s, 0, 13, wfs#KR, string(" YOU ARE DEAD!"))
      wfs.beep
      scoreDead[s]++
      writeDeadScore(s)
    elseif (speed[s]<10 and speed[s]>3)
      wfs.repChar(s, 0, 13, wfs#GR, 32, 40)
      wfs.write(s, 0, 13, wfs#GR, string(" YOU CRASHED!"))
      wfs.beep
      wfs.beep
      scoreCrash[s]++
      writeCrashScore(s)
    else
      wfs.repChar(s, 0, 13, wfs#RG, 32, 40)
      wfs.write(s, 0, 13, wfs#RG, string(" SUCCESS! YOU LANDED AND SURVIVED!"))
      wfs.beep
      wfs.beep
      wfs.beep
      scoreLand[s]++
      writeLandScore(s)
    wfs.repChar(s, 0, 14, wfs#KG, 32, 40)   
    wfs.write(s, 0, 14, wfs#KG, @ExitStart)
    state[s] :=-1
    
PRI writeAllscores(s)
  writeDeadScore(s)
  writeCrashScore(s)
  writeLandScore(s)
  
PRI writeDeadScore(s) | position
  position := (s*72) + (9*36) + 26
  fileNumber6(@ScoreMap, position, scoreDead[s])

PRI writeCrashScore(s) | position
  position := (s*72) + (9*36) + 18
  fileNumber6(@ScoreMap, position, scoreCrash[s])

PRI writeLandScore(s) | position
  position := (s*72) + (9*36) + 10
  fileNumber6(@ScoreMap, position, scoreLand[s])

PRI fileNumber6(pathfile, position, n) | pos, val, ch, sign
   val := n
   if val<0
     sign := "-"
     val := -val
   else
     sign := " "
   pos := position + 6
   repeat 5
     ch := (val // 10) + 48
     val := val / 10
     wfs.pokeFile(pathfile, pos, ch, 0, 0)
     pos--
   wfs.pokeFile(pathfile, pos, sign, 0, 0)
DAT
ExitStart     byte " ESC to EXIT - any other key to START   ",0
StoreScreens  byte "/LANDER02/STORESCR.0"
StoreScreensN byte "0","4",0
StoreTemp     byte "/LANDER02/STORETMP.00"
StoreTempS    byte "0",0
HelpFile      byte "/LANDER02/LANDER02.HLP",0
ScoreMap      byte "/LANDER02/SCOREMAP.MAP",0
DAT
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}             