''
''        Author: Marko Lukat
'' Last modified: 2017/12/05
''       Version: 0.1
''
CON
  columns  = driver#res_x / 16
  rows     = driver#res_y / 32
  bcnt     = columns * rows

  rows_raw = (driver#res_y + 32 - 1) / 32
  bcnt_raw = columns * rows_raw

CON
  #8, BS, TAB, LF, VT, FF, CR, ESC = 27
  
OBJ
  driver: "vga.driver"

VAR
  long  link[driver#res_m]                              ' driver mailbox

  word  scrn[bcnt_raw / 2]                              ' screen buffer (2n aligned)
  word  indx[bcnt_raw / 2]                              ' colour buffer (2n aligned)

  byte  x, y, page, colour                              ' cursor position, page mode, colour
  word  flag  
 
PUB getscrnadr
RETURN @scrn
'' This is not a top level object.

PUB init(config) : c

  link{0} := @scrn{0}                                   ' initial screen and
  link[2] := @indx{0}                                   ' colour index buffer (default palette)
  link[3] := config                                     ' lock/vgrp/vpin

  return driver.init(@link{0})                          ' video driver and pixel generator

PUB setn(n, addr)

  link[n] := addr
  repeat
  while link[n]
  
PUB fill(ch, fb) : c

  repeat bcnt_raw 
    scrn.byte[c] := ch
    indx.byte[c] := fb
    c++

PUB putc(c)

  indx.byte[y * columns + x] := colour                  ' update colour
  scrn.byte[y * columns + x] := c                       ' update character
  if ++x == columns                                       
    x := newline                                        ' CR/LF

PUB punch(ch, kolor, loc) 'loc = 0..599
'Added by JRG. No bounds checking for speed
'display a character in the current color
'c=character to show, loc=location (y*40+x)
  indx.byte[loc] := kolor
  scrn.byte[loc] := ch

PRI newline

  if ++y == rows
    if page                                             ' page/scroll?
      y := 0
      return
    y--

    wordmove(@indx{0}, @indx.byte[columns], constant((bcnt_raw - columns) / 2))
    wordmove(@scrn{0}, @scrn.byte[columns], constant((bcnt_raw - columns) / 2))
    if rows_raw == rows
      wordfill(@indx.byte[constant(bcnt_raw - columns)], colour * $0101, constant(columns / 2))
      wordfill(@scrn.byte[constant(bcnt_raw - columns)], $2020, constant(columns / 2))

PUB str(addr)

  repeat strsize(addr)
    out(byte[addr++])
 
PUB str2(col, row, fb, addr) 'fb=foreground/background colors
  'added by JRG
  bytemove(@scrn+row*40+col, addr, strsize(addr))
  bytefill(@indx+row*40+col, fb, strsize(addr))

PUB str3(col, row, fb, addr, num) 'fb=foreground/background colors
  'added by JRG
  bytemove(@scrn+row*40+col, addr, num)
  bytefill(@indx+row*40+col, fb, num)    

PUB clear(fb) 'fb=foreground/background colors, added by JRG
  fb := fb << 24 | fb << 16 | fb << 8 | fb '$fb > $fb_fb_fb_fb
  longfill(@scrn, $20202020, 150) 'fill with 4 spaces
  longfill(@indx, fb, 150)

PUB out(c) : succ
'' Output a character
''
''     $00 = NUL   clear screen
''     $01 = SOH   home
''     $08 = BS  * backspace
''     $09 = TAB * tab
''     $0A = LF    set X position (X follows)
''     $0B = VT    set Y position (Y follows)
''     $0C = FF  * clear screen       ''     $0D = CR  * return
''     $1B = ESC sequence(s)
''  others = printable characters

  case flag.byte{0}
    $00: case c                                             
           $00..$01,FF:                            
             if c <> $01                           
               wordfill(@indx{0}, colour * $0101, constant(bcnt_raw / 2))
               wordfill(@scrn{0}, $2020, constant(bcnt_raw / 2))
             x := y := 0                           
           $0A, $0B: flag := c << 8 | "="               ' emulate head/tail of ESC=<x><y>
           $0D: x := newline                                     ' CR/LF
           ESC: flag := c                           
           other: putc(c)                            
    ESC: case c  'ESC = 27 or $1B, c = current char
           "s", "S": page := c & $20                    ' page (ESC+s) and scroll mode (ESC+S)
           "=":      succ := constant($0200 | "=")      ' ESC=<x><y>
           "c":      succ := constant($0100 | "c")      ' ESCc<indx>
         flag := succ~
    "=": case flag.byte[1]--
           2..10: flag &= tr(c, @x, columns, flag >> 11)
           other: flag := tr(c, @y, rows, TRUE)
    "c": colour := c 
           flag := 0

PRI tr(c, addr, limit, response)

  if c.byte{0} < 255
    byte[addr] := c.byte{0} // limit

  return not response
            
DAT
  'scrn  word    0[bcnt_raw / 2]                         ' screen buffer (2n aligned)
  'indx  word    0[bcnt_raw / 2]                         ' colour buffer (2n aligned)

DAT